<?php

namespace MetodikaTI\Http\Controllers\Back\Dashboard;

use Illuminate\Http\Request;
use MetodikaTI\Http\Controllers\Controller;
use MetodikaTI\Library\URI;
use MetodikaTI\Institution;
use MetodikaTI\Incidentes;
use MetodikaTI\UsersApp;
use MetodikaTI\Codigos;

use Auth;

class CodigosController extends Controller
{
    public function index(){    
        $user    = Auth::user();
        $codigos = Codigos::where("id_institucion",$user->id_institucion)->get(); 

        $codigosUsers = array();

        foreach ($codigos as $key => $codigo) {
            
            $users_app = UsersApp::where("id_codigo", $codigo->id)->count();                        
            array_push( $codigosUsers, array( "id"       => $codigo->id, 
                                              "codigo"   => $codigo->codigo, 
                                              "name"     => $codigo->name, 
                                              "usuarios" => $users_app ) 
                                            );
        }        
        
        return view('back.codigos.index', array( "codigos" => $codigosUsers, 'permitions' => URI::checkPermitions() ));
    }

    public function editar_codigo(){        
        return view('back.codigos.config', array( 'permitions' => URI::checkPermitions() ));
    }

    public function search_code( Request $request ){

        $response["status"] = true;
    
        $codigos = Codigos::where("codigo",$request->code)->get();
        
        if( count( $codigos ) > 0 ){
            $response["status"]  = false;
            $response["message"] = "El código ya existe.";
        }
        
        return response()->json($response, 200);
    }

    public function search_code_update( Request $request ){

        $response["status"] = true;        
        $codigos = Codigos::where("codigo",$request->code)->where("id","!=",$request->id_codigo)->get();            

        if( count( $codigos ) > 0 ){
            $response["status"]  = false;
            $response["message"] = "El código ya existe.";
        }
        
        return response()->json($response, 200);
    }

    // public function instituciones_code( Request $request ){

    //     $response["status"] = true;
    //     $instituciones = Institution::where("codigo",0)->get();
    //     $response["data"] = $instituciones; 

    //     return response()->json($response, 200);
    // }

    public function add_code( Request $request ){

        $response["status"] = false;
        
        $this->validate(
            $request, 
            [   
                'codigo' => 'required', 
                'name'   => 'required',                                            
            ],
            [   
                'codigo.required' => 'El código es requerido',
                'name.required'   => 'El nombre es requerido',                                
            ]
        );

        
        $user = Auth::user();  
        
        $codigo_q = Codigos::where("name", $request->name)->where("id_institucion", $user->id_institucion)->get();

        if( count( $codigo_q ) > 0 ){
            $response["status"]  = false;
            $response["message"] = "Ya existe un tag con el nombre de ".$request->name;
            return response()->json($response, 200);
        }

        $institucion = Institution::find( $user->id_institucion );
        
        $codigo = new Codigos();

        $codigo->codigo         = $request->codigo;
        $codigo->id_institucion = $institucion->id;
        $codigo->name           = $request->name;        

        if( $codigo->save() ){
            $response["reload"] = true;
            $response["url"] = "config-codigos";
            $response["status"]  = true;
            $response["message"] = "El código se actualizó correctamente.";
        }

        return response()->json($response, 200);

    }

    public function edit_codigo( $id ){
        $codigo = Codigos::find( base64_decode($id) );
        return view('back.codigos.editar', array( "codigo" => $codigo, 'permitions' => URI::checkPermitions() ));
    }

    public function delete_codigo( $id ){

        if( $id ){
            $codigo = Codigos::where("id", base64_decode( $id ) )->delete();

            if( $codigo ){
                $response["status"] = true;
                $response["url"] = route('codigos');
                $response["message"] = "El código se elimino correctamente";
            }else{
                $response["status"] = false;
                $response["message"] = "¡Uy! Algo salió mal. No se encontró el código.";
            }


        }else{
            $response["message"] = "¡Uy! Algo salió mal. No se pudo eliminar el código.";
        }

        return response()->json($response, 200);

    }

    public function update_codigo( Request $request, $id ){

        $response["status"] = false;

        $this->validate(
            $request, 
            [   
                'codigo' => 'required', 
                'name'   => 'required',                                            
            ],
            [   
                'codigo.required' => 'El código es requerido',
                'name.required'   => 'El nombre es requerido',                                
            ]
        );

        $codigos          = Codigos::find(base64_decode($id));
        $codigos->codigo  = $request->codigo;
        $codigos->name    = $request->name;

        if( $codigos->save() ){
            //se eedita en el incidente por que se guarda el string del codigo
            Incidentes::where("id_codigo", base64_decode($id))->update(["codigo" => $request->codigo ]);

            $response["reload"] = true;            
            $response["url"] = route('codigos');
            $response["status"]  = true;
            $response["message"] = "Los datos se actualizáron correctamente."; 

        }else{
            $response["status"]  = false;
            $response["message"] = "¡Uy! Algo salió mal. No se pudo actualizar el teléfono.";
        }

        return response()->json($response, 200);

    }
    
}
