<?php

namespace MetodikaTI\Http\Controllers\Back\Dashboard;

use Illuminate\Http\Request;
use MetodikaTI\Http\Controllers\Controller;
use MetodikaTI\Library\URI;
use MetodikaTI\Comunicados;
use MetodikaTI\Institution;
use MetodikaTI\ComunicadosCodigos;
use MetodikaTI\Codigos;
use MetodikaTI\UsersApp;
use MetodikaTI\ComunicadosVistas;
use MetodikaTI\NotificacionesUsuario;
use MetodikaTI\Library\Pastora;
use DB;
use Auth;

class ComunicadosController extends Controller
{
    public function index(){
        $user = Auth::user();
        $comunicados = Comunicados::where("id_institucion", $user->id_institucion)->get();    
        return view('back.comunicados.index', array( "comunicados" => $comunicados, 'permitions' => URI::checkPermitions() ));
    }

    public function nuevo_comunicado(){
        $user = Auth::user();
        $codigos = Codigos::where("id_institucion", $user->id_institucion)->get();        
        return view('back.comunicados.create', array( 'codigos' => $codigos, 'permitions' => URI::checkPermitions() ));
    }

    public function editar_comunicado( Request $request, $id ){
        
        $comunicado = Comunicados::find( base64_decode($id) );
        $tipos = array( array( "num" => 0, "name" => "Ninguno" ), array( "num" => 1, "name" => "Informativo" ) );
        // $dirigidos = array( array( "num" => 0, "name" => "Ninguno" ), array( "num" => 1, "name" => "Todos" ) );
        $user    = Auth::user();
        $codigos = Codigos::where("id_institucion", $user->id_institucion)->get();
        $codios_seleccionados = ComunicadosCodigos::where("id_comunicado", $comunicado->id)->pluck("id_codigo")->toArray();
        $selecionados = $this->selectComunicadoEdit( $codigos, $codios_seleccionados );
        error_log( "CODIGOS SELECCIONADOS => ". print_r($selecionados, true) );
        return view('back.comunicados.editar', array( "selecionados" => $selecionados, "comunicado" => $comunicado, "tipos" => $tipos, "codigos" => $codigos,"codios_seleccionados" => $codios_seleccionados ));

    }

    private function selectComunicadoEdit( $codigos, $codios_seleccionados ){
        $todos = 0;
        $select = array();
        foreach( $codigos as $codigo ){
            // $status = $this->seleccionados( $codigo, $codios_seleccionados );                   
            array_push( $select, array( "codigo" => $codigo->codigo, "id" => $codigo->id, "select" => $this->seleccionados( $codigo, $codios_seleccionados ) ) );                        
        } 

        $todos = $this->selectTodos( $codios_seleccionados );

        if( $todos ){
            array_push( $select, array( "codigo" => "Todos", "id" => 0, "select" => true ) );
        }
        
        return $select;        
    }

    private function selectTodos( $codios_seleccionados ){
        foreach ($codios_seleccionados as $key => $value) {            
            if( $value == 0 ){
                return true;
            }
        }
    }

    private function seleccionados( $codigo, $codigos ){
        
        foreach($codigos as $key => $item  ){
            if( $item == $codigo->id){                            
                return true;
            }

            if( $codigo->id_codigo === 0 ){                      
                return true;
            }
        }

        return false;
    }
    

    public function delete_comunicado( $id ){

        if( $id ){
            $codigo = Comunicados::where("id", base64_decode( $id ) )->delete();

            if( $codigo ){
                ComunicadosCodigos::where("id_comunicado",base64_decode($id))->delete();
                $response["status"] = true;
                $response["url"] = route('comunicados');
                $response["message"] = "El comunicado se eliminó correctamente";
            }else{
                $response["status"] = false;
                $response["message"] = "¡Uy! Algo salió mal. No se encontró el comunicado.";
            }


        }else{
            $response["message"] = "¡Uy! Algo salió mal. No se pudo eliminar el comunicado.";
        }
        
        return response()->json($response, 200);
    }


    public function update_comunicado( Request $request, $id ){

        $response["status"] = false;

        $this->validate(
            $request, 
            [   
                'titulo' => 'required', 
                'desc'   => 'required',                                            
            ],
            [   
                'titulo.required' => 'El campo título es requerido.',
                'desc.required'   => 'El campo descripción es requerido.',                                
            ]
        );

        $user = Auth::user();
        $comunicado = Comunicados::find( base64_decode( $id ) );

        if($request->hasFile("video")){
            
            $file_name = md5(date("Y-m-d H:i:s")).".".$request->file('video')->getClientOriginalExtension();

            $path = "assets/comunicados/videos/".md5($user->id_institucion);
            if (!file_exists($path)) {
                mkdir($path, 0775, true);
            }

            $request->file('video')->move("assets/comunicados/videos/".md5($user->id_institucion), $file_name );

            // if( file_exists($user->profile_image) ){
            //     unlink($user->profile_image);
            // }

            // $user->profile_image = "assets/images/users/".$file_name;

            $comunicado->url_video = "assets/comunicados/videos/".md5($user->id_institucion)."/".$file_name;
        }

        if($request->hasFile("imagen")){

            $file_name = md5(date("Y-m-d H:i:s")).".".$request->file('imagen')->getClientOriginalExtension();

            $path = "assets/comunicados/images/".md5($user->id_institucion);
            if (!file_exists($path)) {
                mkdir($path, 0775, true);
            }

            $request->file('imagen')->move("assets/comunicados/images/".md5($user->id_institucion), $file_name );

            $comunicado->url_img = "assets/comunicados/images/".md5($user->id_institucion)."/".$file_name;

        }

        if( $request->url_youtube ){
            // $valida_url = preg_match('/https:\/\/www\.youtube\.com\/watch\?v=[^&]+/', $request->url_youtube);
            // if( !$valida_url ){
            //     $response["message"] = "La URL de YouTube es invalida";
            //     return response()->json($response, 200);
            // }
            $comunicado->url_youtube   = $request->url_youtube;
        }

        if($request->hasFile("pdf")){

            $file_name = md5(date("Y-m-d H:i:s")).".".$request->file('pdf')->getClientOriginalExtension();

            $path = "assets/comunicados/pdf/".md5($user->id_institucion);
            if (!file_exists($path)) {
                mkdir($path, 0775, true);
            }

            $request->file('pdf')->move("assets/comunicados/pdf/".md5($user->id_institucion), $file_name );

            $comunicado->url_pdf = "assets/comunicados/pdf/".md5($user->id_institucion)."/".$file_name;

        }

        

        error_log($request->add_pdf);
        error_log($request->url_video);
        error_log($request->imagen);

        // return
    

        $comunicado->titulo   = $request->titulo;
        $comunicado->tipo     = $request->tipo;
        // $comunicado->dirigido = $request->dirigido;
        $comunicado->texto    = $request->texto;
        $comunicado->descripcion    = $request->desc;
        $comunicado->id_institucion = $user->id_institucion;
        
        if( $comunicado->save() ){

            if( empty( $request->add_pdf ) ){
            
                if($comunicado->url_pdf){
                    if(unlink($comunicado->url_pdf)){
                        error_log( "eliminar pdf" );
                    }else{
                        error_log( "no se eliminar pdf" );
                    }
                    Comunicados::where( "id", base64_decode( $id ) )->update([ "url_pdf" => "" ]);
                }
    
            }

            if( empty( $request->imagen ) ){
            
                if($comunicado->url_img){
                    if(unlink($comunicado->url_img)){
                        error_log( "eliminar imagen" );
                    }else{
                        error_log( "no se eliminar imagen" );
                    }
                    Comunicados::where( "id", base64_decode( $id ) )->update([ "url_img" => "" ]);
                }
    
            }

            if( empty( $request->url_video ) ){
            
                if($comunicado->url_video or $comunicado->url_youtube){

                    if($comunicado->url_video){
                        if(unlink($comunicado->url_video)){
                            error_log( "eliminar imagen" );
                        }else{
                            error_log( "no se eliminar imagen" );
                        }
                    }
                    Comunicados::where( "id", base64_decode( $id ) )->update([ "url_video" => "", "url_youtube" => "" ]);
                }


    
            }

            $comunicado_antes =  ComunicadosCodigos::where("id_comunicado", $comunicado->id )->pluck("id")->toArray();

            ComunicadosVistas::whereIn("id_comunicado_codigos", $comunicado_antes )->delete();

            ComunicadosCodigos::where("id_comunicado", base64_decode( $id ) )->delete();

            $response["status"] = true;
            $response["url"]    = route("comunicados");
            $response["message"] = "El comunicado se actualizó correctamente.";

            foreach( $request->dirigido as $item ){
                $com_codigos = new ComunicadosCodigos();
                $com_codigos->id_codigo     = $item;
                $com_codigos->id_comunicado = $comunicado->id;
                $com_codigos->save();

                if( $item == 0 ){                                        

                    //SE BUSCARA A TODOS LOS USUARIOS QUE PERTENESCAN A ESTA INSTITUCIÓN                    
                    $comunicados = Comunicados::find( $comunicado->id );
                    $codigos     = Codigos::where( "id_institucion", $comunicados->id_institucion )->pluck("id")->toArray();
                    $users_app   = UsersApp::whereIn( "id_codigo", $codigos )->get();

                    $ids_users = array();

                    foreach ($users_app as $key => $user_app) {

                        $comunicado_views = new ComunicadosVistas();
                        $comunicado_views->id_comunicado_codigos = $com_codigos->id;
                        $comunicado_views->id_user = $user_app->id;
                        $comunicado_views->visto = 0;
                        $comunicado_views->save();

                        array_push($ids_users, $user_app->id);
                    }

                    // $notificaciones = NotificacionesUsuario::whereIn( "id_user", $ids_users )->pluck("token")->toArray();

                    // if( count( $notificaciones ) > 0 ){

                    //     // $response["RESPONSE"]["UUID"] = $notificaciones->token; 
                    //     $response["RESPONSE"]["TEXTO_NOTIFICACION"] = "Tienes un comunicado nuevo";
                    //     $response["RESPONSE"]["TITLE"] = "Notificación";
                    //     $response["RESPONSE"]["ids"] = $notificaciones;
                        
                    //     //Si se pasa un true se envia una sola notificacion con los ids
                    //     Pastora::sendPushNotification( $response, true );

                    // }


                    break;

                }else{

                    
                    // USUARIOS QUE PERTENECEN A UN CODIGO
                    $users_app = UsersApp::where( "id_codigo", $item )->get();                                                        
                    $ids_users = array();

                    foreach ($users_app as $key => $user_app) {

                        $comunicado_views = new ComunicadosVistas();
                        $comunicado_views->id_comunicado_codigos = $com_codigos->id;
                        $comunicado_views->id_user = $user_app->id;
                        $comunicado_views->visto = 0;
                        $comunicado_views->save(); 
                        
                        array_push($ids_users, $user_app->id);
                        
                    }
                    

                    // $notificaciones = NotificacionesUsuario::whereIn( "id_user", $ids_users )->pluck("token")->toArray();

                    // if( count($notificaciones) > 0 ){

                    //     error_log("crea comunicado =>". print_r( $notificaciones, true ) );
    
                    //     $response["RESPONSE"]["TEXTO_NOTIFICACION"] = "Tienes un comunicado nuevo";
                    //     $response["RESPONSE"]["TITLE"] = "Notificación";
                    //     $response["RESPONSE"]["ids"]   = $notificaciones;
    
                    //     //Si se pasa un true se envia una sola notificacion con los ids
                    //     Pastora::sendPushNotification( $response, true );

                    // }

                }                

                // $com_codigos->id_codigo = $item->
            }

        }else{
            $response["mensaje"] = "¡Uy! Algo salió mal. No se pudo registrar el comunicado.";
        }

        return response()->json($response, 200);

    }

    public function save_comunicado( Request $request ){

        
        $response["status"] = false;         
        
        $this->validate(
            $request, 
            [   
                'titulo' => 'required', 
                'desc'   => 'required'                                           
            ],
            [   
                'titulo.required' => 'El campo título es requerido.',
                'desc.required'   => 'El campo descripción es requerido.',                             
            ]
        );


        DB::beginTransaction();                    

        $user = Auth::user();
        $comunicado = new Comunicados();        

        if($request->hasFile("video")){
            
            $file_name = md5(date("Y-m-d H:i:s")).".".$request->file('video')->getClientOriginalExtension();

            $path = "assets/comunicados/videos/".md5($user->id_institucion);
            if (!file_exists($path)) {
                mkdir($path, 0775, true);
            }

            $request->file('video')->move("assets/comunicados/videos/".md5($user->id_institucion), $file_name );

            // if( file_exists($user->profile_image) ){
            //     unlink($user->profile_image);
            // }

            // $user->profile_image = "assets/images/users/".$file_name;

            $comunicado->url_video = "assets/comunicados/videos/".md5($user->id_institucion)."/".$file_name;
        }

        if($request->hasFile("imagen")){

            $file_name = md5(date("Y-m-d H:i:s")).".".$request->file('imagen')->getClientOriginalExtension();

            $path = "assets/comunicados/images/".md5($user->id_institucion);
            if (!file_exists($path)) {
                mkdir($path, 0775, true);
            }

            $request->file('imagen')->move("assets/comunicados/images/".md5($user->id_institucion), $file_name );

            $comunicado->url_img = "assets/comunicados/images/".md5($user->id_institucion)."/".$file_name;

        }

        if($request->hasFile("pdf")){

            $file_name = md5(date("Y-m-d H:i:s")).".".$request->file('pdf')->getClientOriginalExtension();

            $path = "assets/comunicados/pdf/".md5($user->id_institucion);
            if (!file_exists($path)) {
                mkdir($path, 0775, true);
            }

            $request->file('pdf')->move("assets/comunicados/pdf/".md5($user->id_institucion), $file_name );

            $comunicado->url_pdf = "assets/comunicados/pdf/".md5($user->id_institucion)."/".$file_name;

        }

        if( $request->url_youtube ){
            // $valida_url = preg_match('/https:\/\/www\.youtube\.com\/watch\?v=[^&]+/', $request->url_youtube);
            // if( !$valida_url ){
            //     $response["message"] = "La URL de YouTube es invalida";
            //     return response()->json($response, 200);
            // }
            $comunicado->url_youtube   = $request->url_youtube;
        }
    

        $comunicado->titulo   = $request->titulo;
        $comunicado->tipo     = $request->tipo;
        // $comunicado->dirigido = $request->dirigido;
        $comunicado->texto    = $request->texto;
        $comunicado->descripcion    = $request->desc;
        $comunicado->id_institucion = $user->id_institucion;
    
        if( $comunicado->save() ){
            $response["status"]  = true;
            $response["url"]     = route("comunicados");
            $response["message"] = "El comunicado se guardó correctamente.";

            
            foreach( $request->dirigido as $item ){                

                $com_codigos = new ComunicadosCodigos();
                $com_codigos->id_codigo     = $item;
                $com_codigos->id_comunicado = $comunicado->id;
                $com_codigos->save();                                                  

                if( $item == 0 ){
                    
                    //SE BUSCARA A TODOS LOS USUARIOS QUE PERTENESCAN A ESTA INSTITUCIÓN                    
                    $comunicados = Comunicados::find( $comunicado->id );
                    $codigos     = Codigos::where( "id_institucion", $comunicados->id_institucion )->pluck("id")->toArray();
                    $users_app   = UsersApp::whereIn( "id_codigo", $codigos )->get();

                    $ids_users = array();

                    foreach ($users_app as $key => $user_app) {

                        $comunicado_views = new ComunicadosVistas();
                        $comunicado_views->id_comunicado_codigos = $com_codigos->id;
                        $comunicado_views->id_user = $user_app->id;
                        $comunicado_views->visto = 0;
                        $comunicado_views->save();

                        array_push($ids_users, $user_app->id);
                    }

                    $notificaciones = NotificacionesUsuario::whereIn( "id_user", $ids_users )->pluck("token")->toArray();

                    if( count( $notificaciones ) > 0 ){

                        // $response["RESPONSE"]["UUID"] = $notificaciones->token; 
                        $response["RESPONSE"]["TEXTO_NOTIFICACION"] = "Tienes un comunicado nuevo";
                        $response["RESPONSE"]["TITLE"] = "Notificación";
                        $response["RESPONSE"]["ids"] = $notificaciones;
                        $response["RESPONSE"]["DATA"] = array( "tipo" => "comunicado", "id_comunicado" => $comunicado->id );
                        
                        //Si se pasa un true se envia una sola notificacion con los ids
                        Pastora::sendPushNotification( $response, true );

                    }
                
                }else{

                    
                    // USUARIOS QUE PERTENECEN A UN CODIGO
                    $users_app = UsersApp::where( "id_codigo", $item )->get();                                                        
                    $ids_users = array();

                    foreach ($users_app as $key => $user_app) {

                        $comunicado_views = new ComunicadosVistas();
                        $comunicado_views->id_comunicado_codigos = $com_codigos->id;
                        $comunicado_views->id_user = $user_app->id;
                        $comunicado_views->visto = 0;
                        $comunicado_views->save(); 
                        
                        array_push($ids_users, $user_app->id);
                        
                    }
                    

                    $notificaciones = NotificacionesUsuario::whereIn( "id_user", $ids_users )->pluck("token")->toArray();

                    if( count($notificaciones) > 0 ){

                        // error_log("crea comunicado =>". print_r( $notificaciones, true ) );
    
                        $response["RESPONSE"]["TEXTO_NOTIFICACION"] = "Tienes un comunicado nuevo";
                        $response["RESPONSE"]["TITLE"] = "Notificación";
                        $response["RESPONSE"]["ids"]   = $notificaciones;
                        $response["RESPONSE"]["DATA"] = array( "tipo" => "comunicado", "id_comunicado" => $comunicado->id );
    
                        //Si se pasa un true se envia una sola notificacion con los ids
                        Pastora::sendPushNotification( $response, true );

                    }

                }

            }

        }else{
            $response["mensaje"] = "¡Uy! Algo salió mal. No se pudo registrar el comunicado.";;
        }

        DB::commit();

        return response()->json($response, 200);
    }

}
